@@ -4,6 +4,7 @@ from typing import Optional
 
 from pycountry import countries, languages  # type: ignore[import]
 
+
 DEFAULT_LANGUAGE = "en"
 DEFAULT_COUNTRY = "US"
 DEFAULT_LANGUAGE_CODE = f"{DEFAULT_LANGUAGE}_{DEFAULT_COUNTRY}"
@@ -28,10 +29,13 @@ class Country:
                 c.alpha_3,
                 c.numeric,
                 c.name,
-                getattr(c, "official_name", c.name)
+                getattr(c, "official_name", c.name),
             )
-        except (LookupError, KeyError):
-            raise LookupError(f"Invalid country code: {country}")
+        except LookupError as err:
+            raise LookupError(f"Invalid country code: {country}") from err
+
+    def __hash__(self):
+        return hash((self.alpha2, self.alpha3, self.numeric, self.name, self.official_name))
 
     def __eq__(self, other):
         return (
@@ -46,7 +50,7 @@ class Country:
             self.alpha3,
             self.numeric,
             self.name,
-            self.official_name
+            self.official_name,
         )
 
 
@@ -73,10 +77,13 @@ class Language:
                 getattr(lang, "alpha_2", ""),
                 lang.alpha_3,
                 lang.name,
-                getattr(lang, "bibliographic", "")
+                getattr(lang, "bibliographic", ""),
             )
-        except (LookupError, KeyError):
-            raise LookupError(f"Invalid language code: {language}")
+        except LookupError as err:
+            raise LookupError(f"Invalid language code: {language}") from err
+
+    def __hash__(self):
+        return hash((self.alpha2, self.alpha3, self.name, self.bibliographic))
 
     def __eq__(self, other):
         return (
@@ -90,7 +97,7 @@ class Language:
             self.alpha2,
             self.alpha3,
             self.name,
-            self.bibliographic
+            self.bibliographic,
         )
 
 
@@ -118,9 +125,9 @@ class Localization:
 
     def _set_language_code(self, language_code):
         is_system_locale = language_code is None
-        if language_code is None:
+        if is_system_locale:
             try:
-                language_code, _ = locale.getdefaultlocale()
+                language_code, _ = locale.getlocale()
             except ValueError:
                 language_code = None
             if language_code is None or language_code == "C":
@@ -131,13 +138,12 @@ class Localization:
             self.language, self.country = self._parse_locale_code(language_code)
             self._language_code = language_code
         except LookupError:
-            if is_system_locale:
-                # If the system locale returns an invalid code, use the default
-                self.language = self.get_language(DEFAULT_LANGUAGE)
-                self.country = self.get_country(DEFAULT_COUNTRY)
-                self._language_code = DEFAULT_LANGUAGE_CODE
-            else:
+            if not is_system_locale:
                 raise
+            # If the system locale returns an invalid code, use the default
+            self.language = self.get_language(DEFAULT_LANGUAGE)
+            self.country = self.get_country(DEFAULT_COUNTRY)
+            self._language_code = DEFAULT_LANGUAGE_CODE
         log.debug(f"Language code: {self._language_code}")
 
     def equivalent(self, language: Optional[str] = None, country: Optional[str] = None) -> bool:
