@@ -1,10 +1,10 @@
 import logging
 import warnings
+from datetime import timezone
 from inspect import currentframe, getframeinfo
 from io import StringIO
 from pathlib import Path
 from typing import Iterable, Tuple, Type
-from unittest.mock import patch
 
 import freezegun
 import pytest
@@ -13,34 +13,62 @@ from streamlink import logger
 from streamlink.exceptions import StreamlinkDeprecationWarning, StreamlinkWarning
 
 
-@pytest.fixture
+@pytest.fixture()
 def output():
     return StringIO()
 
 
-@pytest.fixture
-def log(request, output: StringIO):
+@pytest.fixture()
+def logfile(tmp_path: Path) -> str:
+    return str(tmp_path / "log.txt")
+
+
+@pytest.fixture()
+def log(request: pytest.FixtureRequest, monkeypatch: pytest.MonkeyPatch, output: StringIO):
     params = getattr(request, "param", {})
     params.setdefault("format", "[{name}][{levelname}] {message}")
     params.setdefault("style", "{")
+
+    if "logfile" in request.fixturenames:
+        params["filename"] = request.getfixturevalue("logfile")
+
     fakeroot = logging.getLogger("streamlink.test")
-    with patch("streamlink.logger.root", fakeroot):
-        logger.basicConfig(stream=output, **params)
-        yield fakeroot
-        logger.capturewarnings(False)
+
+    monkeypatch.setattr("streamlink.logger.root", fakeroot)
+    monkeypatch.setattr("streamlink.utils.times.LOCAL", timezone.utc)
+
+    handler = logger.basicConfig(stream=output, **params)
+    assert isinstance(handler, logging.Handler)
+
+    yield fakeroot
+
+    logger.capturewarnings(False)
+
+    handler.close()
+    fakeroot.removeHandler(handler)
+    assert not fakeroot.handlers
 
 
 class TestLogging:
-    @pytest.fixture
-    def log_failure(self, request, log: logging.Logger, output: StringIO):
+    @pytest.fixture()
+    def log_failure(
+        self,
+        request: pytest.FixtureRequest,
+        monkeypatch: pytest.MonkeyPatch,
+        log: logging.Logger,
+        output: StringIO,
+    ):
         params = getattr(request, "param", {})
+
         root = logging.getLogger("streamlink")
-        with pytest.raises(Exception) as cm:
-            with patch("streamlink.logger.root", root):
-                logger.basicConfig(stream=output, **params)
+        monkeypatch.setattr("streamlink.logger.root", root)
+
+        with pytest.raises(Exception) as cm:  # noqa: PT011
+            logger.basicConfig(stream=output, **params)
+
         return cm.value
 
-    @pytest.mark.parametrize("name,level", [
+    @pytest.mark.parametrize(("name", "level"), [
         ("none", logger.NONE),
         ("critical", logger.CRITICAL),
         ("error", logger.ERROR),
@@ -87,7 +115,7 @@ class TestLogging:
         log.debug("test")
         assert output.getvalue() == "[test][debug] test\n"
 
-    @pytest.mark.parametrize("loglevel,calllevel,expected", [
+    @pytest.mark.parametrize(("loglevel", "calllevel", "expected"), [
         (logger.DEBUG, logger.TRACE, ""),
         (logger.TRACE, logger.TRACE, "[test][trace] test\n"),
         (logger.TRACE, logger.DEBUG, "[test][debug] test\n"),
@@ -101,7 +129,7 @@ class TestLogging:
         assert output.getvalue() == expected
 
     # https://github.com/streamlink/streamlink/issues/4862
-    @pytest.mark.parametrize("level,levelname", [
+    @pytest.mark.parametrize(("level", "levelname"), [
         (logger.TRACE, "trace"),
         (logger.ALL, "all"),
     ])
@@ -115,7 +143,7 @@ class TestLogging:
             ("test_logger", levelname, "bar"),
         ]
 
-    @pytest.mark.parametrize("level,expected", [
+    @pytest.mark.parametrize(("level", "expected"), [
         (logger.DEBUG, ""),
         (logger.INFO, "[test][info] foo\n[test][info] bar\n"),
     ])
@@ -152,12 +180,19 @@ class TestLogging:
 
     @freezegun.freeze_time("2000-01-02T03:04:05.123456Z")
     @pytest.mark.parametrize("log", [
-        {"format": "[{asctime}][{name}][{levelname}] {message}", "datefmt": "%H:%M:%S.%f"},
+        {"format": "[{asctime}][{name}][{levelname}] {message}", "datefmt": "%H:%M:%S.%f%z"},
     ], indirect=True)
     def test_datefmt_custom(self, log: logging.Logger, output: StringIO):
         log.setLevel("info")
         log.info("test")
-        assert output.getvalue() == "[03:04:05.123456][test][info] test\n"
+        assert output.getvalue() == "[03:04:05.123456+0000][test][info] test\n"
+
+    def test_logfile(self, logfile: str, log: logging.Logger, output: StringIO):
+        log.setLevel("info")
+        log.info("Hello world, Γειά σου Κόσμε, こんにちは世界")
+        log.handlers[0].flush()
+        with open(logfile, "r", encoding="utf-8") as fh:
+            assert fh.read() == "[test][info] Hello world, Γειά σου Κόσμε, こんにちは世界\n"
 
 
 class TestCaptureWarnings:
@@ -181,7 +216,7 @@ class TestCaptureWarnings:
         assert output.getvalue() == ""
 
     @pytest.mark.parametrize("log", [{"capture_warnings": True}], indirect=["log"])
-    @pytest.mark.parametrize("warning,expected,origin", [
+    @pytest.mark.parametrize(("warning", "expected", "origin"), [
         (("Test warning", UserWarning), "[warnings][userwarning] Test warning\n", True),
         (("Test warning", DeprecationWarning), "[warnings][deprecationwarning] Test warning\n", True),
         (("Test warning", FutureWarning), "[warnings][futurewarning] Test warning\n", True),
