@@ -1,17 +1,18 @@
 import re
-import unittest
-import warnings
+from contextlib import nullcontext
 from pathlib import Path
 from socket import AF_INET, AF_INET6
-from unittest.mock import Mock, call, patch
+from typing import Dict
+from unittest.mock import Mock
 
 import pytest
-import requests_mock
+import requests_mock as rm
 import urllib3
 from requests.adapters import HTTPAdapter
 
 import tests.plugin
 from streamlink.exceptions import NoPluginError, StreamlinkDeprecationWarning
+from streamlink.options import Options
 from streamlink.plugin import HIGH_PRIORITY, LOW_PRIORITY, NO_PRIORITY, NORMAL_PRIORITY, Plugin, pluginmatcher
 from streamlink.plugin.api.http_session import TLSNoDHAdapter
 from streamlink.session import Streamlink
@@ -25,132 +26,155 @@ PATH_TESTPLUGINS_OVERRIDE = PATH_TESTPLUGINS / "override"
 _original_allowed_gai_family = urllib3.util.connection.allowed_gai_family  # type: ignore[attr-defined]
 
 
-class EmptyPlugin(Plugin):
-    def _get_streams(self):
-        pass  # pragma: no cover
+class TestLoadPlugins:
+    @pytest.fixture(autouse=True)
+    def caplog(self, caplog: pytest.LogCaptureFixture) -> pytest.LogCaptureFixture:
+        caplog.set_level(1, "streamlink")
+        return caplog
 
-
-# TODO: rewrite using pytest
-class TestSession(unittest.TestCase):
-    mocker: requests_mock.Mocker
-
-    def setUp(self):
-        self.mocker = requests_mock.Mocker()
-        self.mocker.register_uri(requests_mock.ANY, requests_mock.ANY, text="")
-        self.mocker.start()
-
-    def tearDown(self):
-        self.mocker.stop()
-        Streamlink.resolve_url.cache_clear()
-
-    def subject(self, load_plugins=True):
-        session = Streamlink()
-        if load_plugins:
-            session.load_plugins(str(PATH_TESTPLUGINS))
-            session.load_plugins(str(PATH_TESTPLUGINS_OVERRIDE))
-
-        return session
-
-    # ----
-
-    def test_load_plugins(self):
-        session = self.subject()
+    def test_load_plugins(self, caplog: pytest.LogCaptureFixture, session: Streamlink):
+        session.load_plugins(str(PATH_TESTPLUGINS))
         plugins = session.get_plugins()
-        assert "testplugin" in plugins
-        assert "testplugin_missing" not in plugins
-        assert "testplugin_invalid" not in plugins
+        assert list(plugins.keys()) == ["testplugin"]
+        assert plugins["testplugin"].__name__ == "TestPlugin"
+        assert plugins["testplugin"].__module__ == "streamlink.plugins.testplugin"
+        assert caplog.records == []
+
+    def test_load_plugins_override(self, caplog: pytest.LogCaptureFixture, session: Streamlink):
+        session.load_plugins(str(PATH_TESTPLUGINS))
+        session.load_plugins(str(PATH_TESTPLUGINS_OVERRIDE))
+        plugins = session.get_plugins()
+        assert list(plugins.keys()) == ["testplugin"]
+        assert plugins["testplugin"].__name__ == "TestPluginOverride"
+        assert plugins["testplugin"].__module__ == "streamlink.plugins.testplugin"
+        assert [(record.name, record.levelname, record.message) for record in caplog.records] == [
+            (
+                "streamlink.session",
+                "debug",
+                f"Plugin testplugin is being overridden by {PATH_TESTPLUGINS_OVERRIDE / 'testplugin.py'}",
+            ),
+        ]
 
     def test_load_plugins_builtin(self):
-        session = self.subject()
+        session = Streamlink()
         plugins = session.get_plugins()
         assert "twitch" in plugins
         assert plugins["twitch"].__module__ == "streamlink.plugins.twitch"
 
-    @patch("streamlink.session.log")
-    def test_load_plugins_override(self, mock_log):
-        session = self.subject()
+    @pytest.mark.parametrize(("side_effect", "raises", "logs"), [
+        pytest.param(
+            ImportError,
+            nullcontext(),
+            [
+                (
+                    "streamlink.session",
+                    "error",
+                    f"Failed to load plugin testplugin from {PATH_TESTPLUGINS}",
+                    True,
+                ),
+                (
+                    "streamlink.session",
+                    "error",
+                    f"Failed to load plugin testplugin_invalid from {PATH_TESTPLUGINS}",
+                    True,
+                ),
+                (
+                    "streamlink.session",
+                    "error",
+                    f"Failed to load plugin testplugin_missing from {PATH_TESTPLUGINS}",
+                    True,
+                ),
+            ],
+            id="ImportError",
+        ),
+        pytest.param(
+            SyntaxError,
+            pytest.raises(SyntaxError),
+            [],
+            id="SyntaxError",
+        ),
+    ])
+    def test_load_plugins_failure(
+        self,
+        monkeypatch: pytest.MonkeyPatch,
+        caplog: pytest.LogCaptureFixture,
+        side_effect: Exception,
+        raises: nullcontext,
+        logs: list,
+    ):
+        monkeypatch.setattr("streamlink.session.Streamlink.load_builtin_plugins", Mock())
+        monkeypatch.setattr("streamlink.session.load_module", Mock(side_effect=side_effect))
+        session = Streamlink()
+        with raises:
+            session.load_plugins(str(PATH_TESTPLUGINS))
+        assert session.get_plugins() == {}
+        assert [(record.name, record.levelname, record.message, bool(record.exc_info)) for record in caplog.records] == logs
+
+
+class _EmptyPlugin(Plugin):
+    def _get_streams(self):
+        pass  # pragma: no cover
+
+
+class TestResolveURL:
+    @pytest.fixture(autouse=True)
+    def _load_builtins(self, session: Streamlink):
+        session.load_plugins(str(PATH_TESTPLUGINS))
+
+    @pytest.fixture(autouse=True)
+    def requests_mock(self, requests_mock: rm.Mocker):
+        return requests_mock
+
+    def test_resolve_url(self, recwarn: pytest.WarningsRecorder, session: Streamlink):
         plugins = session.get_plugins()
-        assert "testplugin" in plugins
-        assert "testplugin_override" not in plugins
-        assert plugins["testplugin"].__name__ == "TestPluginOverride"
-        assert plugins["testplugin"].__module__ == "streamlink.plugins.testplugin"
-        assert mock_log.debug.call_args_list == [
-            call(f"Plugin testplugin is being overridden by {PATH_TESTPLUGINS_OVERRIDE / 'testplugin.py'}"),
-        ]
-
-    @patch("streamlink.session.load_module")
-    @patch("streamlink.session.log")
-    def test_load_plugins_importerror(self, mock_log, mock_load_module):
-        mock_load_module.side_effect = ImportError()
-        session = self.subject()
-        assert not session.get_plugins()
-        assert len(mock_log.exception.call_args_list) > 0
-
-    @patch("streamlink.session.load_module")
-    @patch("streamlink.session.log")
-    def test_load_plugins_syntaxerror(self, mock_log, mock_load_module):
-        mock_load_module.side_effect = SyntaxError()
-        with pytest.raises(SyntaxError):
-            self.subject()
-
-    def test_resolve_url(self):
-        session = self.subject()
-        plugins = session.get_plugins()
-
-        with warnings.catch_warnings(record=True) as record_warnings:
-            pluginname, pluginclass, resolved_url = session.resolve_url("http://test.se/channel")
+        pluginname, pluginclass, resolved_url = session.resolve_url("http://test.se/channel")
 
         assert issubclass(pluginclass, Plugin)
         assert pluginclass is plugins["testplugin"]
         assert resolved_url == "http://test.se/channel"
         assert hasattr(session.resolve_url, "cache_info"), "resolve_url has a lookup cache"
-        assert record_warnings == []
+        assert recwarn.list == []
 
-    def test_resolve_url__noplugin(self):
-        session = self.subject()
-        self.mocker.get("http://invalid2", status_code=301, headers={"Location": "http://invalid3"})
+    def test_resolve_url__noplugin(self, requests_mock: rm.Mocker, session: Streamlink):
+        requests_mock.get("http://invalid2", status_code=301, headers={"Location": "http://invalid3"})
 
         with pytest.raises(NoPluginError):
             session.resolve_url("http://invalid1")
         with pytest.raises(NoPluginError):
             session.resolve_url("http://invalid2")
 
-    def test_resolve_url__redirected(self):
-        session = self.subject()
-        plugins = session.get_plugins()
-        self.mocker.head("http://redirect1", status_code=501)
-        self.mocker.get("http://redirect1", status_code=301, headers={"Location": "http://redirect2"})
-        self.mocker.head("http://redirect2", status_code=301, headers={"Location": "http://test.se/channel"})
+    def test_resolve_url__redirected(self, requests_mock: rm.Mocker, session: Streamlink):
+        requests_mock.request("HEAD", "http://redirect1", status_code=501)
+        requests_mock.request("GET", "http://redirect1", status_code=301, headers={"Location": "http://redirect2"})
+        requests_mock.request("GET", "http://redirect2", status_code=301, headers={"Location": "http://test.se/channel"})
+        requests_mock.request("GET", "http://test.se/channel", content=b"")
 
+        plugins = session.get_plugins()
         pluginname, pluginclass, resolved_url = session.resolve_url("http://redirect1")
         assert issubclass(pluginclass, Plugin)
         assert pluginclass is plugins["testplugin"]
         assert resolved_url == "http://test.se/channel"
 
-    def test_resolve_url_no_redirect(self):
-        session = self.subject()
+    def test_resolve_url_no_redirect(self, session: Streamlink):
         plugins = session.get_plugins()
-
         pluginname, pluginclass, resolved_url = session.resolve_url_no_redirect("http://test.se/channel")
         assert issubclass(pluginclass, Plugin)
         assert pluginclass is plugins["testplugin"]
         assert resolved_url == "http://test.se/channel"
 
-    def test_resolve_url_no_redirect__noplugin(self):
-        session = self.subject()
+    def test_resolve_url_no_redirect__noplugin(self, session: Streamlink):
         with pytest.raises(NoPluginError):
             session.resolve_url_no_redirect("http://invalid")
 
-    def test_resolve_url_scheme(self):
+    def test_resolve_url_scheme(self, session: Streamlink):
         @pluginmatcher(re.compile("http://insecure"))
-        class PluginHttp(EmptyPlugin):
+        class PluginHttp(_EmptyPlugin):
             pass
 
         @pluginmatcher(re.compile("https://secure"))
-        class PluginHttps(EmptyPlugin):
+        class PluginHttps(_EmptyPlugin):
             pass
 
-        session = self.subject(load_plugins=False)
         session.plugins = {
             "insecure": PluginHttp,
             "secure": PluginHttps,
@@ -167,32 +191,31 @@ class TestSession(unittest.TestCase):
             session.resolve_url("http://secure")
         assert session.resolve_url("https://secure")[1] is PluginHttps
 
-    def test_resolve_url_priority(self):
+    def test_resolve_url_priority(self, session: Streamlink):
         @pluginmatcher(priority=HIGH_PRIORITY, pattern=re.compile(
             "https://(high|normal|low|no)$",
         ))
-        class HighPriority(EmptyPlugin):
+        class HighPriority(_EmptyPlugin):
             pass
 
         @pluginmatcher(priority=NORMAL_PRIORITY, pattern=re.compile(
             "https://(normal|low|no)$",
         ))
-        class NormalPriority(EmptyPlugin):
+        class NormalPriority(_EmptyPlugin):
             pass
 
         @pluginmatcher(priority=LOW_PRIORITY, pattern=re.compile(
             "https://(low|no)$",
         ))
-        class LowPriority(EmptyPlugin):
+        class LowPriority(_EmptyPlugin):
             pass
 
         @pluginmatcher(priority=NO_PRIORITY, pattern=re.compile(
             "https://(no)$",
         ))
-        class NoPriority(EmptyPlugin):
+        class NoPriority(_EmptyPlugin):
             pass
 
-        session = self.subject(load_plugins=False)
         session.plugins = {
             "high": HighPriority,
             "normal": NormalPriority,
@@ -216,57 +239,13 @@ class TestSession(unittest.TestCase):
         with pytest.raises(NoPluginError):
             session.resolve_url_no_redirect("no")
 
-    def test_resolve_deprecated(self):
-        @pluginmatcher(priority=LOW_PRIORITY, pattern=re.compile(
-            "https://low",
-        ))
-        class LowPriority(EmptyPlugin):
-            pass
 
-        class DeprecatedNormalPriority(EmptyPlugin):
-            # noinspection PyUnusedLocal
-            @classmethod
-            def can_handle_url(cls, url):
-                return True
+class TestStreams:
+    @pytest.fixture(autouse=True)
+    def _load_builtins(self, session: Streamlink):
+        session.load_plugins(str(PATH_TESTPLUGINS))
 
-        class DeprecatedHighPriority(DeprecatedNormalPriority):
-            # noinspection PyUnusedLocal
-            @classmethod
-            def priority(cls, url):
-                return HIGH_PRIORITY
-
-        session = self.subject(load_plugins=False)
-        session.plugins = {
-            "empty": EmptyPlugin,
-            "low": LowPriority,
-            "dep-normal-one": DeprecatedNormalPriority,
-            "dep-normal-two": DeprecatedNormalPriority,
-            "dep-high": DeprecatedHighPriority,
-        }
-
-        with pytest.warns() as recwarn:
-            plugin = session.resolve_url_no_redirect("low")[1]
-
-        assert plugin is DeprecatedHighPriority
-        assert [(record.category, str(record.message)) for record in recwarn.list] == [
-            (StreamlinkDeprecationWarning, "Resolved plugin dep-normal-one with deprecated can_handle_url API"),
-            (StreamlinkDeprecationWarning, "Resolved plugin dep-high with deprecated can_handle_url API"),
-        ]
-
-    def test_options(self):
-        session = self.subject()
-        session.set_option("test_option", "option")
-        assert session.get_option("test_option") == "option"
-        assert session.get_option("non_existing") is None
-
-        assert session.get_plugin_option("testplugin", "a_option") == "default"
-        session.set_plugin_option("testplugin", "another_option", "test")
-        assert session.get_plugin_option("testplugin", "another_option") == "test"
-        assert session.get_plugin_option("non_existing", "non_existing") is None
-        assert session.get_plugin_option("testplugin", "non_existing") is None
-
-    def test_streams(self):
-        session = self.subject()
+    def test_streams(self, session: Streamlink):
         streams = session.streams("http://test.se/channel")
 
         assert "best" in streams
@@ -276,9 +255,14 @@ class TestSession(unittest.TestCase):
         assert isinstance(streams["http"], HTTPStream)
         assert isinstance(streams["hls"], HLSStream)
 
-    def test_streams_stream_types(self):
-        session = self.subject()
+    def test_streams_options(self, session: Streamlink):
+        streams = session.streams("http://test.se/fromoptions", Options({"streamurl": "http://foo/"}))
 
+        assert sorted(streams.keys()) == ["best", "fromoptions", "worst"]
+        assert isinstance(streams["fromoptions"], HTTPStream)
+        assert streams["fromoptions"].url == "http://foo/"
+
+    def test_stream_types(self, session: Streamlink):
         streams = session.streams("http://test.se/channel", stream_types=["http", "hls"])
         assert isinstance(streams["480p"], HTTPStream)
         assert isinstance(streams["480p_hls"], HLSStream)
@@ -287,9 +271,7 @@ class TestSession(unittest.TestCase):
         assert isinstance(streams["480p"], HLSStream)
         assert isinstance(streams["480p_http"], HTTPStream)
 
-    def test_streams_stream_sorting_excludes(self):
-        session = self.subject()
-
+    def test_stream_sorting_excludes(self, session: Streamlink):
         streams = session.streams("http://test.se/channel", sorting_excludes=[])
         assert "best" in streams
         assert "worst" in streams
@@ -329,87 +311,117 @@ class TestSession(unittest.TestCase):
         assert "vod_alt" in streams
         assert "vod_alt2" in streams
 
-    def test_set_and_get_locale(self):
-        session = Streamlink()
-        session.set_option("locale", "en_US")
-        assert session.localization.country.alpha2 == "US"
-        assert session.localization.language.alpha2 == "en"
-        assert session.localization.language_code == "en_US"
 
-    @patch("streamlink.session.HTTPSession")
-    def test_interface(self, mock_httpsession):
-        adapter_http = Mock(poolmanager=Mock(connection_pool_kw={}))
-        adapter_https = Mock(poolmanager=Mock(connection_pool_kw={}))
-        adapter_foo = Mock(poolmanager=Mock(connection_pool_kw={}))
-        mock_httpsession.return_value = Mock(adapters={
-            "http://": adapter_http,
-            "https://": adapter_https,
-            "foo://": adapter_foo,
-        })
-        session = self.subject(load_plugins=False)
+def test_options(session: Streamlink):
+    session.set_option("test_option", "option")
+    assert session.get_option("test_option") == "option"
+    assert session.get_option("non_existing") is None
+
+
+def test_options_locale(monkeypatch: pytest.MonkeyPatch, session: Streamlink):
+    monkeypatch.setattr("locale.getlocale", lambda: ("C", None))
+    assert session.get_option("locale") is None
+
+    localization = session.localization
+    assert localization.explicit is False
+    assert localization.language_code == "en_US"
+    assert localization.country.alpha2 == "US"
+    assert localization.country.name == "United States"
+    assert localization.language.alpha2 == "en"
+    assert localization.language.name == "English"
+
+    session.set_option("locale", "de_DE")
+    assert session.get_option("locale") == "de_DE"
+
+    localization = session.localization
+    assert localization.explicit is True
+    assert localization.language_code == "de_DE"
+    assert localization.country.alpha2 == "DE"
+    assert localization.country.name == "Germany"
+    assert localization.language.alpha2 == "de"
+    assert localization.language.name == "German"
+
+
+class TestOptionsInterface:
+    @pytest.fixture()
+    def adapters(self, monkeypatch: pytest.MonkeyPatch):
+        adapters = {
+            scheme: Mock(poolmanager=Mock(connection_pool_kw={}))
+            for scheme in ("http://", "https://", "foo://")
+        }
+        monkeypatch.setattr("streamlink.session.HTTPSession", Mock(return_value=Mock(adapters=adapters)))
+
+        return adapters
+
+    def test_options_interface(self, adapters: Dict[str, Mock], session: Streamlink):
         assert session.get_option("interface") is None
 
         session.set_option("interface", "my-interface")
-        assert adapter_http.poolmanager.connection_pool_kw == {"source_address": ("my-interface", 0)}
-        assert adapter_https.poolmanager.connection_pool_kw == {"source_address": ("my-interface", 0)}
-        assert adapter_foo.poolmanager.connection_pool_kw == {}
+        assert adapters["http://"].poolmanager.connection_pool_kw == {"source_address": ("my-interface", 0)}
+        assert adapters["https://"].poolmanager.connection_pool_kw == {"source_address": ("my-interface", 0)}
+        assert adapters["foo://"].poolmanager.connection_pool_kw == {}
         assert session.get_option("interface") == "my-interface"
 
         session.set_option("interface", None)
-        assert adapter_http.poolmanager.connection_pool_kw == {}
-        assert adapter_https.poolmanager.connection_pool_kw == {}
-        assert adapter_foo.poolmanager.connection_pool_kw == {}
+        assert adapters["http://"].poolmanager.connection_pool_kw == {}
+        assert adapters["https://"].poolmanager.connection_pool_kw == {}
+        assert adapters["foo://"].poolmanager.connection_pool_kw == {}
         assert session.get_option("interface") is None
 
-    @patch("streamlink.session.urllib3_util_connection", allowed_gai_family=_original_allowed_gai_family)
-    def test_ipv4_ipv6(self, mock_urllib3_util_connection):
-        session = self.subject(load_plugins=False)
-        assert session.get_option("ipv4") is False
-        assert session.get_option("ipv6") is False
-        assert mock_urllib3_util_connection.allowed_gai_family is _original_allowed_gai_family
-
-        session.set_option("ipv4", True)
-        assert session.get_option("ipv4") is True
-        assert session.get_option("ipv6") is False
-        assert mock_urllib3_util_connection.allowed_gai_family is not _original_allowed_gai_family
-        assert mock_urllib3_util_connection.allowed_gai_family() is AF_INET
-
-        session.set_option("ipv4", False)
-        assert session.get_option("ipv4") is False
-        assert session.get_option("ipv6") is False
-        assert mock_urllib3_util_connection.allowed_gai_family is _original_allowed_gai_family
-
-        session.set_option("ipv6", True)
-        assert session.get_option("ipv4") is False
-        assert session.get_option("ipv6") is True
-        assert mock_urllib3_util_connection.allowed_gai_family is not _original_allowed_gai_family
-        assert mock_urllib3_util_connection.allowed_gai_family() is AF_INET6
-
-        session.set_option("ipv6", False)
-        assert session.get_option("ipv4") is False
-        assert session.get_option("ipv6") is False
-        assert mock_urllib3_util_connection.allowed_gai_family is _original_allowed_gai_family
-
-        session.set_option("ipv4", True)
-        session.set_option("ipv6", False)
-        assert session.get_option("ipv4") is True
-        assert session.get_option("ipv6") is False
-        assert mock_urllib3_util_connection.allowed_gai_family is _original_allowed_gai_family
-
-    def test_http_disable_dh(self):
-        session = self.subject(load_plugins=False)
-        assert isinstance(session.http.adapters["https://"], HTTPAdapter)
-        assert not isinstance(session.http.adapters["https://"], TLSNoDHAdapter)
-
-        session.set_option("http-disable-dh", True)
-        assert isinstance(session.http.adapters["https://"], TLSNoDHAdapter)
-
-        session.set_option("http-disable-dh", False)
-        assert isinstance(session.http.adapters["https://"], HTTPAdapter)
-        assert not isinstance(session.http.adapters["https://"], TLSNoDHAdapter)
+        # doesn't raise
+        session.set_option("interface", None)
 
 
-class TestSessionOptionHttpProxy:
+def test_options_ipv4_ipv6(monkeypatch: pytest.MonkeyPatch, session: Streamlink):
+    mock_urllib3_util_connection = Mock(allowed_gai_family=_original_allowed_gai_family)
+    monkeypatch.setattr("streamlink.session.urllib3_util_connection", mock_urllib3_util_connection)
+
+    assert session.get_option("ipv4") is False
+    assert session.get_option("ipv6") is False
+    assert mock_urllib3_util_connection.allowed_gai_family is _original_allowed_gai_family
+
+    session.set_option("ipv4", True)
+    assert session.get_option("ipv4") is True
+    assert session.get_option("ipv6") is False
+    assert mock_urllib3_util_connection.allowed_gai_family is not _original_allowed_gai_family
+    assert mock_urllib3_util_connection.allowed_gai_family() is AF_INET
+
+    session.set_option("ipv4", False)
+    assert session.get_option("ipv4") is False
+    assert session.get_option("ipv6") is False
+    assert mock_urllib3_util_connection.allowed_gai_family is _original_allowed_gai_family
+
+    session.set_option("ipv6", True)
+    assert session.get_option("ipv4") is False
+    assert session.get_option("ipv6") is True
+    assert mock_urllib3_util_connection.allowed_gai_family is not _original_allowed_gai_family
+    assert mock_urllib3_util_connection.allowed_gai_family() is AF_INET6
+
+    session.set_option("ipv6", False)
+    assert session.get_option("ipv4") is False
+    assert session.get_option("ipv6") is False
+    assert mock_urllib3_util_connection.allowed_gai_family is _original_allowed_gai_family
+
+    session.set_option("ipv4", True)
+    session.set_option("ipv6", False)
+    assert session.get_option("ipv4") is True
+    assert session.get_option("ipv6") is False
+    assert mock_urllib3_util_connection.allowed_gai_family is _original_allowed_gai_family
+
+
+def test_options_http_disable_dh(session: Streamlink):
+    assert isinstance(session.http.adapters["https://"], HTTPAdapter)
+    assert not isinstance(session.http.adapters["https://"], TLSNoDHAdapter)
+
+    session.set_option("http-disable-dh", True)
+    assert isinstance(session.http.adapters["https://"], TLSNoDHAdapter)
+
+    session.set_option("http-disable-dh", False)
+    assert isinstance(session.http.adapters["https://"], HTTPAdapter)
+    assert not isinstance(session.http.adapters["https://"], TLSNoDHAdapter)
+
+
+class TestOptionsHttpProxy:
     @pytest.fixture()
     def _no_deprecation(self, recwarn: pytest.WarningsRecorder):
         yield
